<?php

namespace Drupal\order\Controller;

use Drupal\Core\Ajax\HtmlCommand;
use Drupal\Core\Ajax\InvokeCommand;
use Drupal\Core\Ajax\ReplaceCommand;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Url;
use Drupal\order\Plugin\CheckoutManager;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\PageCache\ResponsePolicyInterface;
use Drupal\Core\Session\SessionManagerInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Drupal\order\Entity\Product;
use Drupal\order\Entity\Order;
use Drupal\order\Entity\Store;
use Drupal\order\Entity\Item;
use Symfony\Component\HttpFoundation\RequestStack;
use Drupal\Core\TempStore\PrivateTempStoreFactory;

/**
 * The primary controller for managing orders.
 */
class OrderController extends ControllerBase {

  /**
   * Drupal\Core\Entity\EntityTypeManagerInterface definition.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Drupal\Core\Messenger\MessengerInterface definition.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * Drupal\Core\PageCache\ResponsePolicyInterface definition.
   *
   * @var \Drupal\Core\PageCache\ResponsePolicyInterface
   */
  protected $pageCacheKillSwitch;

  /**
   * Drupal\Core\Session\SessionManagerInterface definition.
   *
   * @var \Drupal\Core\Session\SessionManagerInterface
   */
  protected $sessionManager;

  /**
   * Symfony\Component\EventDispatcher\EventSubscriberInterface definition.
   *
   * @var \Symfony\Component\EventDispatcher\EventSubscriberInterface
   */
  protected $redirectResponseSubscriber;

  /**
   * Symfony\Component\HttpFoundation\RequestStack definition.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * User private temp store.
   *
   * @var \Drupal\Core\TempStore\PrivateTempStore
   */
  protected $store;

  /**
   * Checkout Manager.
   *
   * @var \Drupal\order\Plugin\CheckoutManager
   */
  protected $checkoutManager;

  /**
   * Constructs a new OrderController object.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, MessengerInterface $messenger, ResponsePolicyInterface $page_cache_kill_switch, SessionManagerInterface $session_manager, EventSubscriberInterface $redirect_response_subscriber, RequestStack $request_stack, AccountInterface $current_user, PrivateTempStoreFactory $temp_store_factory, CheckoutManager $checkout_manager) {
    $this->entityTypeManager = $entity_type_manager;
    $this->messenger = $messenger;
    $this->pageCacheKillSwitch = $page_cache_kill_switch;
    $this->sessionManager = $session_manager;
    $this->redirectResponseSubscriber = $redirect_response_subscriber;
    $this->requestStack = $request_stack;
    $this->currentUser = $current_user;
    $this->tempStoreFactory = $temp_store_factory;
    $this->checkoutManager = $checkout_manager;

    $this->store = $this->tempStoreFactory->get('order.cart');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('messenger'),
      $container->get('page_cache_kill_switch'),
      $container->get('session_manager'),
      $container->get('redirect_response_subscriber'),
      $container->get('request_stack'),
      $container->get('current_user'),
      $container->get('tempstore.private'),
      $container->get('plugin.manager.order.checkout')
    );
  }

  /**
   * Add a product to the order.
   */
  public function addToOrder(Product $product) {
    // Get the store the product belongs to.
    $store = $product->getStore();
    $labels = $store->getOverrides();

    // Retrieve order.
    $order = $this->getOrder($store);
    // Add product to order via an Item.
    $this->setItem($order, $product);
    // Prevent caching of this page.
    $this->pageCacheKillSwitch->trigger();

    $use_ajax = $store->get('use_ajax');

    if ($use_ajax) {
      $response = new AjaxResponse();
      $selector = '#ajax-add-message-' . $product->uuid() . ' .ajax-message:contains()';
      $content = [
        '#type' => 'markup',
        '#markup' => $labels['added_text'],
        '#prefix' => '<div class="ajax-message">',
        '#suffix' => '</div>',
      ];
      $settings = [];
      $command = new ReplaceCommand($selector, $content, $settings);

      $count_selector = '.block-order span.count';
      $count_content = [
        '#type' => 'markup',
        '#markup' => $this->orderCount($store),
      ];
      $count_command = new HtmlCommand($count_selector, $count_content, []);

      $text_selector = '#ajax-add-message-' . $product->uuid() . ' button';
      $text_command = new HtmlCommand($text_selector, $labels['added_button'], []);

      $changed_command = new InvokeCommand(
        '#ajax-add-message-' . $product->uuid() . ' a',
        'addClass',
        ['product-added']
      );

      return $response
        ->addCommand($text_command)
        ->addCommand($count_command)
        ->addCommand($changed_command)
        ->addCommand($command);
    }
    else {
      return $this->redirect('order.viewCart', ['store' => $store->id()]);
    }
  }

  /**
   * Remove an item from the order.
   */
  public function removeFromOrder(Item $item) {
    $item->delete();
    $order = $item->getOrder();
    $store = $order->getStore();
    return $this->redirect('order.viewCart', ['store' => $store->id()]);
  }

  /**
   * Get the order entity based on the Store we're working with.
   */
  public function getOrder($store, $create = TRUE) {
    // Start a manual session for anonymous users.
    if ($this->currentUser->isAnonymous() && !isset($_SESSION['order_in_progress']) && $create) {
      $_SESSION['order_in_progress'] = TRUE;
      $this->sessionManager->start();
    }

    // Retrieve an order from the current user's session.
    $order_data = $this->store->get('order.cart');

    // Get Store ID.
    if (is_string($store)) {
      $store_id = $store;
    }
    elseif (is_object($store) && $store instanceof Store) {
      $store_id = $store->id();
    }

    if (!empty($order_data) && !empty($order_data[$store_id]['order'])) {
      $order = $this->entityTypeManager()->getStorage('order')->load($order_data[$store_id]['order']);
    }
    else {
      $order_data = [];
    }

    // Create an order if one was not retrieved.
    if (empty($order) || !$order instanceof Order && $create) {
      $order = $this->entityTypeManager()->getStorage('order')->create([
        'name' => $this->sessionManager->getId(),
        'status' => 1,
        'store' => $store,
      ]);
      $order->save();

      // Why are we merging the array?
      $order_data = array_merge($order_data, [$store_id => ['order' => $order->id()]]);

      $this->store->set('order.cart', $order_data);
    }

    return $order;
  }

  /**
   * Get the items for an order.
   */
  public function getItems(Order $order) {
    return $this->entityTypeManager->getStorage('item')->loadByProperties(['order' => $order->id()]);
  }

  /**
   * Creates or updates an Item entity based on the Order and Product.
   */
  public function setItem(Order $order, Product $product) {
    // Retrieve an existing item for this product and order.
    $items = $this->entityTypeManager->getStorage('item')->loadByProperties([
      'order' => $order->id(),
      'product' => $product->id(),
    ]);
    if (!empty($items)) {
      // Check store or product settings for quantity and update if needed.
      $item = array_pop($items);
    }
    else {
      // If there are no existing items, create one.
      $item = $this->entityTypeManager->getStorage('item')->create([
        'order' => $order->id(),
        'product' => $product->id(),
        'quantity' => 1,
      ]);
      $item->save();
    }
    return $item;
  }

  /**
   * Get a data array for a given Order.
   */
  public function getOrderData(Order $order) {
    $data = [];

    $data['order'] = $order;
    $data['store'] = $order->getStore();
    $data['items'] = [];

    $items = $this->getItems($order);

    foreach ($items as $key => $item) {
      /** @var Drupal\order\Entity\Product $product */
      $product = $item->get('product')
        ->first()
        ->get('entity')
        ->getTarget();
      if ($product <> NULL && ($product = $product->getValue()) instanceof Product) {
        $parent = $product->getParentEntity();
        $data['items'][$key]['item'] = $item;
        $data['items'][$key]['product'] = $product;
        $data['items'][$key]['parent'] = $parent;
      }
    }

    return $data;
  }

  /**
   * View an Order.
   */
  public function viewOrder(Store $store) {
    $build = [];
    if (!empty($store) && $store instanceof Store) {
      $order = $this->getOrder($store);
      $data = $this->getOrderData($order);
      $build['order'] = [
        '#theme' => 'order',
        '#order' => $data['order'],
        '#store' => $data['store'],
        '#items' => $data['items'],
        '#cache' => [
          'max-age' => 0,
        ],
        '#attached' => [
          'library' => [
            'order/order',
          ],
        ],
      ];
    }
    return $build;
  }

  /**
   * Callback to return "View Order" title.
   */
  public function getOrderViewOrderTitle(Store $store) {
    $overrides = $store->getOverrides();
    return $overrides['order_title'];
  }

  /**
   * Callback for Complete Order Title.
   */
  public function getCompleteOrderTitle(Store $store) {
    $overrides = $store->getOverrides();
    return $overrides['order_title'];
  }

  /**
   * Retrieve the checkout plugin's form and return it.
   */
  public function completeOrder(Store $store) {
    $order = $this->getOrder($store);
    $plugin = $store->get('checkout_plugin');
    $checkoutPlugin = $this->checkoutManager->createInstance($plugin);
    $form = $checkoutPlugin->buildForm($order);
    return $form;
  }

  /**
   * Submit the order checkout form.
   */
  public static function orderCheckoutSubmitForm(array $form, FormStateInterface $form_state) {
    // Get normalized values.
    $data = $form_state->get('processedCheckoutData');

    $build_info = $form_state->getBuildInfo();
    if ($build_info['args'] && $build_info['args'][0] instanceof Order) {
      /** @var \Drupal\order\Entity\Order $order */
      $order = $build_info['args'][0];
    }
    else {
      return;
    }
    $store = $order->getStore();
    $status = OrderController::processOrder($order, $data);
    if ($status) {
      // Processed successfully, remove order from temp store.
      $temp_store = \Drupal::service('tempstore.private')->get('order.cart');
      $temp_store->delete('order.cart');

      // Update order status.
      if ($confirmation = $store->get('confirmation')) {
        $url_object = \Drupal::service('path.validator')->getUrlIfValid($confirmation);
        if ($url_object instanceof Url) {
          $form_state->setRedirect($url_object->getRouteName(), $url_object->getRouteParameters());
        }
      }
    }
  }

  /**
   * Process the order data through the process plugin.
   */
  public static function processOrder(Order $order, $data) {
    $store = $order->getStore();

    // Pass values to process plugin.
    $plugin = $store->get('process_plugin');
    $orderProcessPlugin = \Drupal::service('plugin.manager.order.process')->createInstance($plugin);

    return $orderProcessPlugin->processOrder($order, $data);
  }

  /**
   * Return the count of the number of items in an order.
   */
  public function orderCount($store) {
    $order = $this->getOrder($store, FALSE);
    if (!empty($order) && $order instanceof Order) {
      $items = $this->getItems($order);
    }
    else {
      $items = [];
    }
    return count($items);
  }

}
