<?php

namespace Drupal\order\Form;

use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\order\Plugin\CheckoutManager;
use Drupal\order\Plugin\OrderProcessManager;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Base configuration for the Store form.
 */
class StoreForm extends EntityForm {

  /**
   * Checkout Manager plugin.
   *
   * @var \Drupal\order\Plugin\CheckoutManager
   */
  protected $checkoutManager;

  /**
   * Order Process Manager plugin.
   *
   * @var \Drupal\order\Plugin\OrderProcessManager
   */
  protected $processManager;

  /**
   * StoreForm constructor.
   */
  public function __construct(CheckoutManager $checkout_manager, OrderProcessManager $process_manager) {
    $this->checkoutManager = $checkout_manager;
    $this->processManager = $process_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('plugin.manager.order.checkout'),
      $container->get('plugin.manager.order.process')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $form = parent::form($form, $form_state);

    $store = $this->entity;
    $form['label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Label'),
      '#maxlength' => 255,
      '#default_value' => $store->label(),
      '#description' => $this->t("Label for the Store."),
      '#required' => TRUE,
    ];

    $form['id'] = [
      '#type' => 'machine_name',
      '#default_value' => $store->id(),
      '#machine_name' => [
        'exists' => '\Drupal\order\Entity\Store::load',
      ],
      '#disabled' => !$store->isNew(),
    ];

    $form['base_path'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Base path'),
      '#maxlength' => 255,
      '#description' => $this->t('The base path or return path for the store. This is the path users will be redirected to when clicking "continue shopping" or similar from the order page.'),
      '#default_value' => $store->get('base_path'),
    ];

    $form['use_ajax'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Use ajax for adding and removing items from the order.'),
      '#description' => $this->t('If disabled actions will redirect to the view order page.'),
      '#default_value' => $store->get('use_ajax') ? $store->get('use_ajax') : 0,
    ];

    $form['order'] = [
      '#type' => 'details',
      '#title' => $this->t('Order settings'),
      '#open' => TRUE,
    ];

    $checkout_plugins = $this->checkoutManager->getDefinitions();
    $options = [NULL => $this->t('Select one')];
    foreach ($checkout_plugins as $plugin) {
      $options[$plugin['id']] = $plugin['label'];
    }

    $form['order']['checkout_plugin'] = [
      '#type' => 'select',
      '#title' => $this->t('Checkout plugin'),
      '#description' => $this->t('Select a checkout plugin to use for this store.'),
      '#options' => $options,
      '#default_value' => $store->get('checkout_plugin'),
    ];

    $process_plugins = $this->processManager->getDefinitions();
    $options = [NULL => $this->t('Select one')];
    foreach ($process_plugins as $plugin) {
      $options[$plugin['id']] = $plugin['label'];
    }

    $ajax_wrapper = 'process-plugin-ajax-wrapper';

    $form['order']['process_plugin'] = [
      '#type' => 'select',
      '#title' => $this->t('Process plugin'),
      '#description' => $this->t('Select a process plugin to use for this store.'),
      '#options' => $options,
      '#default_value' => $store->get('process_plugin'),
      '#ajax' => [
        'callback' => [$this, 'processPluginChange'],
        'event' => 'change',
        'wrapper' => $ajax_wrapper,
      ],
    ];

    $form['order']['process'] = [
      '#type' => 'container',
      '#attributes' => [
        'id' => $ajax_wrapper,
      ],
    ];

    $plugin = !empty($form_state->getValue('process_plugin')) ? $form_state->getValue('process_plugin') : $store->get('process_plugin');
    if (!empty($plugin)) {
      $orderProcessPlugin = $this->processManager->createInstance($plugin);
      $orderProcessPlugin->settingsForm($form, $form_state, $store->id());
    }

    $form['order']['confirmation'] = [
      '#type' => 'textfield',
      '#title' => 'Confirmation page',
      '#default_value' => $store->get('confirmation'),
    ];

    $form['label_overrides'] = [
      '#type' => 'details',
      '#title' => $this->t('Label overrides'),
      '#tree' => TRUE,
      '#open' => TRUE,
    ];

    $overrides = $store->getOverrides();

    $form['label_overrides']['process_name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Process name'),
      '#default_value' => $overrides['process_name'],
    // Order, Cart.
    ];
    $form['label_overrides']['process_name_plural'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Process name, plural'),
      '#default_value' => $overrides['process_name_plural'],
    // Orders, Carts.
    ];
    $form['label_overrides']['order_title'] = [
      '#type' => 'textfield',
      '#title' => $this->t('View order title'),
      '#default_value' => $overrides['order_title'],
    ];
    $form['label_overrides']['empty_order'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Empty order'),
      '#default_value' => $overrides['empty_order'],
    ];
    $form['label_overrides']['view_order_button'] = [
      '#type' => 'textfield',
      '#title' => $this->t('View order button'),
      '#default_value' => $overrides['view_order_button'],
    ];
    $form['label_overrides']['add_button'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Add button'),
      '#default_value' => $overrides['add_button'],
    ];
    $form['label_overrides']['added_button'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Add button (Added state)'),
      '#default_value' => $overrides['added_button'],
    ];
    $form['label_overrides']['added_text'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Added text'),
      '#description' => $this->t('Text displayed beneath the add button on ajax requests'),
      '#default_value' => $overrides['added_text'],
    ];
    $form['label_overrides']['remove_text'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Remove item label'),
      '#default_value' => $overrides['remove_text'],
    ];
    $form['label_overrides']['update_button'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Update item button'),
      '#default_value' => $overrides['update_button'],
    ];
    $form['label_overrides']['update_order_button'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Update order button'),
      '#default_value' => $overrides['update_order_button'],
    ];
    $form['label_overrides']['update_order_message'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Update order message'),
      '#default_value' => $overrides['update_order_message'],
    ];
    $form['label_overrides']['continue_text'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Continue label'),
      '#default_value' => $overrides['continue_text'],
    ];
    $form['label_overrides']['checkout_title'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Checkout title'),
      '#default_value' => $overrides['checkout_title'],
    ];
    $form['label_overrides']['checkout_text'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Checkout text'),
      '#default_value' => $overrides['checkout_text'],
    ];
    $form['label_overrides']['checkout_button'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Checkout button'),
      '#default_value' => $overrides['checkout_button'],
    ];

    return $form;
  }

  /**
   * AJAX callback for process plugins.
   */
  public function processPluginChange(array $form, FormStateInterface $form_state) {
    return $form['order']['process'];
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    $store = $this->entity;
    $status = $store->save();

    if ($plugin = $form_state->getValue('process_plugin')) {
      $orderProcessPlugin = $this->processManager->createInstance($plugin);
      $orderProcessPlugin->settingsSubmit($form, $form_state);
    }

    switch ($status) {
      case SAVED_NEW:
        drupal_set_message($this->t('Created the %label Store.', [
          '%label' => $store->label(),
        ]));
        break;

      default:
        drupal_set_message($this->t('Saved the %label Store.', [
          '%label' => $store->label(),
        ]));
    }
    $form_state->setRedirectUrl($store->toUrl('collection'));
  }

}
